@@ -156,15 +156,22 @@ public:
   virtual int bigEndianSamples() const = 0;
 
   // return information about drive
-  virtual const DriveInfo *driveInfo(int showErrorMsg) { return NULL; }
+  virtual const DriveInfo *driveInfo(bool showErrorMsg) { return NULL; }
 
   // returns current writing speed
   virtual int speed() { return speed_; }
   
+  // returns current reading speed
+  virtual int rspeed() { return rspeed_; }
+
   // sets writing speed, returns 0 for OK or 1 for illegal speed,
   // this function may send SCSI commands to the drive
   virtual int speed(int) = 0;
 
+  // sets reading speed, returns 0 for OK or 1 for illegal speed,
+  // this function may send SCSI commands to the drive
+  virtual bool rspeed(int);
+
   // sets/return buffer under run protection setting (if supported by
   // the drive: 1 = enabled, 0 = disbaled
   virtual int bufferUnderRunProtection() const {
@@ -203,6 +210,10 @@ public:
   virtual int rawDataReading() const { return rawDataReading_; }
   virtual void rawDataReading(int f) { rawDataReading_ = f != 0 ? 1 : 0; }
 
+  // Returns/sets mode2 mixed track reading flag
+  virtual int mode2Mixed() const { return mode2Mixed_; }
+  virtual void mode2Mixed(int f) { mode2Mixed_ = f != 0 ? 1 : 0; }
+
   virtual TrackData::SubChannelMode subChanReadMode() const { return subChanReadMode_; }
   virtual void subChanReadMode(TrackData::SubChannelMode m) { subChanReadMode_ = m; }
 
@@ -395,11 +406,13 @@ protected:
   int enableBufferUnderRunProtection_;
   int enableWriteSpeedControl_;
   int speed_;
+  int rspeed_;
   int simulate_;
   int multiSession_;
   int encodingMode_; // mode for encoding data sectors
   int fastTocReading_;
   int rawDataReading_;
+  int mode2Mixed_;
   TrackData::SubChannelMode subChanReadMode_;
   int padFirstPregap_; // used by 'read-toc': defines if the first audio 
                        // track's pre-gap is padded with zeros in the toc-file
