@@ -688,7 +688,7 @@ int GenericMMC::getStartOfSession(long *lba)
 
 static unsigned char leadInOutDataMode(TrackData::Mode mode)
 {
-  unsigned char ret;
+  unsigned char ret = 0;
 
   switch (mode) {
   case TrackData::AUDIO:
@@ -1700,7 +1700,7 @@ int GenericMMC::readSubChannels(TrackData::SubChannelMode sm,
   int retries = 5;
   unsigned char cmd[12];
   int i;
-  long blockLen;
+  long blockLen = 0;
   unsigned long subChanMode = 0;
 
   cmd[0] = 0xbe;  // READ CD
@@ -2523,8 +2523,14 @@ int GenericMMC::readCdTest(long lba, long len, int subChanMode) const
 
 	  chan.type(SubChannel::QMODE1DATA);
 
-	  long pqlba =
-	    Msf(chan.amin(), chan.asec(), chan.aframe()).lba() - 150;
+	  int min = chan.amin();
+	  int sec = chan.asec();
+	  int frac = chan.aframe();
+
+	  if ((frac >= 0 && frac < 75) &&
+          (sec >= 0 && sec < 60) &&
+          (min >= 0)) {
+	  long pqlba = Msf(min, sec, frac).lba() - 150;
 
 	  long diff = pqlba - lba;
 	  if (diff < 0)
@@ -2533,6 +2539,7 @@ int GenericMMC::readCdTest(long lba, long len, int subChanMode) const
 	  if (diff < 20) {
 	    pqSubChanBcdOk++;
 	  }
+	  }
 	}
 
 	if (buf[7] < 100 && buf[8] < 60 && buf[9] < 75) {
