@@ -34,14 +34,19 @@
 #include <signal.h>
 #include <pwd.h>
 #include <ctype.h>
+#include <list>
+#include <string>
 
 #include "util.h"
 #include "Toc.h"
 #include "ScsiIf.h"
 #include "CdrDriver.h"
 #include "dao.h"
+#include "port.h"
 #include "Settings.h"
 #include "Cddb.h"
+#include "TempFileManager.h"
+#include "FormatConverter.h"
 
 
 #ifdef UNIXWARE
@@ -64,6 +69,8 @@ static const char *SOURCE_SCSI_DEVICE = NULL;
 static const char *DATA_FILENAME = NULL;
 static const char *CDDB_SERVER_LIST = "freedb.freedb.org freedb.freedb.org:/~cddb/cddb.cgi uk.freedb.org uk.freedb.org:/~cddb/cddb.cgi cz.freedb.org cz.freedb.org:/~cddb/cddb.cgi";
 static const char *CDDB_LOCAL_DB_DIR = NULL;
+static const char *TMP_FILE_DIR = NULL;
+static int READING_SPEED = -1;
 static int WRITING_SPEED = -1;
 static int EJECT = 0;
 static int SWAP = 0;
@@ -74,6 +81,7 @@ static int SESSION = 1; // session for read-toc/read-cd
 static int FAST_TOC = 0; // toc reading without sub-channel analysis
 static int PAUSE = 1; // pause before writing
 static int READ_RAW = 0; // read raw sectors
+static int MODE2_MIXED = 1;
 static int REMOTE_MODE = 0;
 static int REMOTE_FD = -1;
 static int RELOAD = 0;
@@ -92,6 +100,7 @@ static int KEEPIMAGE = 0;
 static int OVERBURN = 0;
 static int BUFFER_UNDER_RUN_PROTECTION = 1;
 static int WRITE_SPEED_CONTROL = 1;
+static bool KEEP = false;
 static bool PRINT_QUERY = false;
 
 static CdrDriver::BlankingMode BLANKING_MODE = CdrDriver::BLANK_MINIMAL;
@@ -173,6 +182,19 @@ static void printVersion()
   message(2, "  Paranoia DAE library - (C) Monty");
   message(2, "");
   message(2, "Check http://cdrdao.sourceforge.net/drives.html#dt for current driver tables.");
+
+  std::list<std::string> list;
+  int num = formatConverter.supportedExtensions(list);
+
+  if (num) {
+    std::string msg = "Format converter enabled for extensions:";
+    std::list<std::string>::iterator i = list.begin();
+    for (;i != list.end(); i++) {
+      msg += " ";
+      msg += (*i);
+    }
+    message(3, msg.c_str());
+  }
   message(1, "");
 }
 
@@ -184,29 +206,35 @@ static void printUsage()
     message(0, "\nUsage: %s <command> [options] [toc-file]", PRGNAME);
     message(0,
 "command:\n"
-"  show-toc  - prints out toc and exits\n"
-"  toc-info  - prints out short toc-file summary\n"
-"  toc-size  - prints total number of blocks for toc\n"
-"  read-toc  - create toc file from audio CD\n"
-"  read-cd   - create toc and rip audio data from CD\n"
-"  read-cddb - contact CDDB server and add data as CD-TEXT to toc-file\n"
-"  show-data - prints out audio data and exits\n"
-"  read-test - reads all audio files and exits\n"
-"  disk-info - shows information about inserted medium\n"
-"  discid    - prints out CDDB information\n"
-"  msinfo    - shows multi session info, output is suited for scripts\n"
-"  unlock    - unlock drive after failed writing\n"
-"  blank     - blank a CD-RW\n"
-"  scanbus   - scan for devices\n"
-"  simulate  - shortcut for 'write --simulate'\n"
-"  write     - writes CD\n"
-"  copy      - copies CD\n");
+"  show-toc   - prints out toc and exits\n"
+"  toc-info   - prints out short toc-file summary\n"
+"  toc-size   - prints total number of blocks for toc\n"
+"  read-toc   - create toc file from audio CD\n"
+"  read-cd    - create toc and rip audio data from CD\n"
+"  read-cddb  - contact CDDB server and add data as CD-TEXT to toc-file\n"
+"  show-data  - prints out audio data and exits\n"
+"  read-test  - reads all audio files and exits\n"
+"  disk-info  - shows information about inserted medium\n"
+"  discid     - prints out CDDB information\n"
+"  msinfo     - shows multi session info, output is suited for scripts\n"
+"  drive-info - shows drive information\n"
+"  unlock     - unlock drive after failed writing\n"
+"  blank      - blank a CD-RW\n"
+"  scanbus    - scan for devices\n"
+"  simulate   - shortcut for 'write --simulate'\n"
+"  write      - writes CD\n"
+"  copy       - copies CD\n");
     
     message (0, "\n Try '%s <command> -h' to get a list of available options\n", PRGNAME);
     break;
     
   case SHOW_TOC:
-    message(0, "\nUsage: %s show-toc [-v #] toc-file\n", PRGNAME);
+    message(0, "\nUsage: %s show-toc [options] toc-file", PRGNAME);
+    message(0,
+"options:\n"
+"  --tmpdir <path>         - sets directory for temporary wav files\n"
+"  --keep                  - keep generated temp wav files after exit\n"
+"  -v #                    - sets verbose level\n");
     break;
     
   case SHOW_DATA:
@@ -238,6 +266,8 @@ static void printUsage()
 "  --buffers #             - sets fifo buffer size (min. 10)\n"
 "  --reload                - reload the disk if necessary for writing\n"
 "  --force                 - force execution of operation\n"
+"  --tmpdir <path>         - sets directory for temporary wav files\n"
+"  --keep                  - keep generated temp wav files after exit\n"
 "  -v #                    - sets verbose level\n"
 "  -n                      - no pause before writing\n",
 	    SCSI_DEVICE);
@@ -270,6 +300,8 @@ static void printUsage()
 "  --buffers #             - sets fifo buffer size (min. 10)\n"
 "  --reload                - reload the disk if necessary for writing\n"
 "  --force                 - force execution of operation\n"
+"  --tmpdir <path>         - sets directory for temporary wav files\n"
+"  --keep                  - keep generated temp wav files after exit\n"
 "  -v #                    - sets verbose level\n"
 "  -n                      - no pause before writing\n",
 	    SCSI_DEVICE);
@@ -285,6 +317,8 @@ static void printUsage()
 "  --session #             - select session\n"
 "  --fast-toc              - do not extract pre-gaps and index marks\n"
 "  --read-raw              - select raw sectors modes for data tracks\n"
+"  --no-mode2-mixed        - don't switch to mode2_mixed\n"
+"  --rspeed <read-speed>   - selects reading speed\n"
 "  --read-subchan <mode>   - defines sub-channel reading mode\n"
 "                            <mode> = rw | rw_raw\n"
 "  --tao-source            - indicate that source CD was written in TAO mode\n"
@@ -335,6 +369,8 @@ static void printUsage()
 "  --session #             - select session\n"
 "  --fast-toc              - do not extract pre-gaps and index marks\n"
 "  --read-raw              - read raw data sectors (including L-EC data)\n"
+"  --no-mode2-mixed        - don't switch to mode2_mixed\n"
+"  --rspeed <read-speed>   - selects reading speed\n"
 "  --read-subchan <mode>   - defines sub-channel reading mode\n"
 "                            <mode> = rw | rw_raw\n"
 "  --tao-source            - indicate that source CD was written in TAO mode\n"
@@ -350,11 +386,22 @@ static void printUsage()
     break;
     
   case TOC_INFO:
-    message(0, "\nUsage: %s toc-info [-v #] toc-file\n", PRGNAME);
+    message(0, "\nUsage: %s toc-info [options] toc-file", PRGNAME);
+    message(0,
+"options:\n"
+"  --tmpdir <path>         - sets directory for temporary wav files\n"
+"  --keep                  - keep generated temp wav files after exit\n"
+"  -v #                    - sets verbose level\n");
     break;
     
   case TOC_SIZE:
-    message(0, "\nUsage: %s toc-size [-v #] toc-file\n", PRGNAME);
+    message(0, "\nUsage: %s toc-size [options] toc-file", PRGNAME);
+    message(0,
+"options:\n"
+"  --tmpdir <path>         - sets directory for temporary wav files\n"
+"  --keep                  - keep generated temp wav files after exit\n"
+"  -v #                    - sets verbose level\n");
+    break;
 
   case BLANK:
     message(0, "\nUsage: %s blank [options]", PRGNAME);
@@ -383,6 +430,17 @@ static void printUsage()
 "  --driver <id>           - force usage of specified driver\n"
 "  --reload                - reload the disk if necessary for writing\n"
 "  --eject                 - ejects cd after unlocking\n"
+"  -v #                    - sets verbose level\n",
+	    SCSI_DEVICE);
+    break;
+    
+  case DRIVE_INFO:
+    message(0, "\nUsage: %s drive-info [options]", PRGNAME);
+    message(0,
+"options:\n"
+"  --device [proto:]{<x,y,z>|device} - sets SCSI device of CD-writer\n"
+"                            (default: %s)\n"
+"  --driver <id>           - force usage of specified driver\n"
 "  -v #                    - sets verbose level\n",
 	    SCSI_DEVICE);
     break;
@@ -398,6 +456,7 @@ static void printUsage()
 "  --source-driver <id>    - force usage of specified driver for source device\n"
 "  --simulate              - just perform a copy simulation\n"
 "  --speed <writing-speed> - selects writing speed\n"
+"  --rspeed <read-speed>   - selects reading speed\n"
 "  --multi                 - session will not be closed\n"
 "  --buffer-under-run-protection #\n"
 "                          - 0: disable buffer under run protection\n"
@@ -530,7 +589,7 @@ static void importSettings(Command cmd)
   }
 
   if (cmd == BLANK || cmd == DISK_INFO || cmd == MSINFO || cmd == UNLOCK ||
-      cmd == DISCID) {
+      cmd == DISCID || cmd == DRIVE_INFO) {
     if ((sval = SETTINGS->getString(SET_WRITE_DRIVER)) != NULL) {
       DRIVER_ID = strdupCC(sval);
     }
@@ -554,7 +613,16 @@ static void importSettings(Command cmd)
 	*ival > 0) {
       CDDB_TIMEOUT = *ival;
     }
+    if ((sval = SETTINGS->getString(SET_TMP_FILE_DIR)) != NULL) {
+        TMP_FILE_DIR = strdupCC(sval);
+    }
   }
+
+  if ((ival = SETTINGS->getInteger(SET_READ_SPEED)) != NULL &&
+      *ival >= 0) {
+    READING_SPEED = *ival;
+  }  
+
 }
 
 static void exportSettings(Command cmd)
@@ -604,7 +672,7 @@ static void exportSettings(Command cmd)
   }
 
   if (cmd == BLANK || cmd == DISK_INFO || cmd == MSINFO || cmd == UNLOCK ||
-      cmd == DISCID) {
+      cmd == DISCID || cmd == DRIVE_INFO) {
     if (DRIVER_ID != NULL)
       SETTINGS->set(SET_WRITE_DRIVER, DRIVER_ID);
     
@@ -627,6 +695,10 @@ static void exportSettings(Command cmd)
       SETTINGS->set(SET_CDDB_TIMEOUT, CDDB_TIMEOUT);
     }
   }
+
+  if (READING_SPEED >= 0) {
+    SETTINGS->set(SET_READ_SPEED, READING_SPEED);
+  }
 }
 
 static int parseCmdline(int argc, char **argv)
@@ -756,6 +828,20 @@ static int parseCmdline(int argc, char **argv)
 	  argc--, argv++;
 	}
       }
+      else if (strcmp((*argv) + 2, "rspeed") == 0) {
+        if (argc < 2) {
+          message(-2, "Missing argument after: %s", *argv);
+          return 1;
+        }
+        else {
+          READING_SPEED = atol(argv[1]);
+          if (READING_SPEED < 0) {
+            message(-2, "Illegal reading speed: %s", argv[1]);
+            return 1;
+          }
+          argc--, argv++;
+        }
+      }
       else if (strcmp((*argv) + 2, "speed") == 0) {
 	if (argc < 2) {
 	  message(-2, "Missing argument after: %s", *argv);
@@ -853,12 +939,18 @@ static int parseCmdline(int argc, char **argv)
       else if (strcmp((*argv) + 2, "read-raw") == 0) {
 	READ_RAW = 1;
       }
+      else if (strcmp((*argv) + 2, "no-mode2-mixed") == 0) {
+	MODE2_MIXED = 0;
+      }
       else if (strcmp((*argv) + 2, "reload") == 0) {
 	RELOAD = 1;
       }
       else if (strcmp((*argv) + 2, "force") == 0) {
 	FORCE = 1;
       }
+      else if (strcmp((*argv) + 2, "keep") == 0) {
+	KEEP = true;
+      }
       else if (strcmp((*argv) + 2, "on-the-fly") == 0) {
 	ON_THE_FLY = 1;
       }
@@ -954,6 +1046,16 @@ static int parseCmdline(int argc, char **argv)
 	  argc--, argv++;
 	}
       }
+      else if (strcmp((*argv) + 2, "tmpdir") == 0) {
+          if (argc < 2) {
+              message(-2, "Missing argument after: %s", *argv);
+              return 1;
+          } else {
+              TMP_FILE_DIR = argv[1];
+              SETTINGS->set(SET_TMP_FILE_DIR, TMP_FILE_DIR);
+              argc--, argv++;
+          }
+      }
       else if (strcmp((*argv) + 2, "cddb-timeout") == 0) {
 	if (argc < 2) {
 	  message(-2, "Missing argument after: %s", *argv);
@@ -1062,6 +1164,26 @@ static int parseCmdline(int argc, char **argv)
   return 0;
 }
 
+// Commit settings to overall system. Export them.
+static void commitSettings(Settings* SETTINGS, const char* settingsPath)
+{
+  if (TMP_FILE_DIR)
+    tempFileManager.setTempDirectory(TMP_FILE_DIR);
+
+  tempFileManager.setKeepTemps(KEEP);
+
+  if (SAVE_SETTINGS && settingsPath != NULL) {
+    // If we're saving our settings, give up root privileges and
+    // exit. The --save option is only compiled in if setreuid() is
+    // available (because it could be used for a local root exploit).
+    if (giveUpRootPrivileges()) {
+      exportSettings(COMMAND);
+      SETTINGS->write(settingsPath);
+    }
+    exit(0);
+  }
+}
+
 // Selects driver for device of 'scsiIf'.
 static CdrDriver *selectDriver(Command cmd, ScsiIf *scsiIf,
 			       const char *driverId)
@@ -1138,14 +1260,14 @@ static CdrDriver *setupDevice(Command cmd, const char *scsiDevice,
   int inquiryFailed = 0;
   int retry = 0;
   int initTries = 2;
-  int ret;
+  int ret = 0;
 
   scsiIf = new ScsiIf(scsiDevice);
 
   switch (scsiIf->init()) {
   case 1:
     message(-2, "Please use option '--device [proto:]bus,id,lun', e.g. "
-            "--device 0,6,0 or --device ATAPI:0,0,0");
+            "--device 0,6,0 or --device ATA:0,0,0");
     delete scsiIf;
     return NULL;
     break;
@@ -1179,6 +1301,13 @@ static CdrDriver *setupDevice(Command cmd, const char *scsiDevice,
   while (initTries > 0) {
     // clear unit attention
     cdr->rezeroUnit(0);
+    if (READING_SPEED >= 0) {
+      if (!cdr->rspeed(READING_SPEED)) {
+        message(-2, "Reading speed %d is not supported by device.",
+                READING_SPEED);
+        exit(1);
+      }
+    }
 
     if (checkReady) {
       retry = 0;
@@ -1205,6 +1334,16 @@ static CdrDriver *setupDevice(Command cmd, const char *scsiDevice,
 	
       cdr->rezeroUnit(0);
 
+      if (READING_SPEED >= 0) {
+        message(0, "Setting reading speed %d.",
+                READING_SPEED); 
+        if (cdr->rspeed(READING_SPEED) != 0) {
+          message(-2, "Reading speed %d is not supported by device.",
+                  READING_SPEED);
+          exit(1);
+        }
+      }
+
       if ((di = cdr->diskInfo()) == NULL) {
 	message(-2, "Cannot get disk information.");
 	delete cdr;
@@ -1250,6 +1389,14 @@ static CdrDriver *setupDevice(Command cmd, const char *scsiDevice,
     }
   }
 
+  if (READING_SPEED >= 0) {
+    if (cdr->rspeed(READING_SPEED) != 0) {
+      message(-2, "Reading speed %d is not supported by device.",
+              READING_SPEED);
+      exit(1);
+    }
+  }
+
   return cdr;
 }
 
@@ -2104,28 +2251,35 @@ int main(int argc, char **argv)
     exit(1);
   }
 
+  commitSettings(SETTINGS, settingsPath);
+
   printVersion();
 
-  if (SAVE_SETTINGS && settingsPath != NULL) {
-    exportSettings(COMMAND);
-    SETTINGS->write(settingsPath);
-  }
-
-
   if (COMMAND != READ_TOC && COMMAND != DISK_INFO && COMMAND != READ_CD &&
       COMMAND != BLANK && COMMAND != SCAN_BUS && COMMAND != UNLOCK &&
       COMMAND != DISCID &&
       COMMAND != COPY_CD && COMMAND != MSINFO && COMMAND != DRIVE_INFO) {
+
+    // Parse TOC file
     toc = Toc::read(TOC_FILE);
 
     if (REMOTE_MODE) {
       unlink(TOC_FILE);
     }
 
-    if (toc == NULL) {
+    // Check and resolve input files paths
+    if (!toc || !toc->resolveFilenames(TOC_FILE)) {
       exitCode = 1; goto fail;
     }
 
+    if (!toc->convertFilesToWav()) {
+      message(-2, "Could not decode audio files from toc file \"%s\".",
+              TOC_FILE);
+      exitCode = 1; goto fail;
+    }
+
+    toc->recomputeLength();
+
     if (COMMAND != SHOW_TOC && COMMAND != READ_CDDB) {
       if (checkToc(toc) != 0) {
 	message(-2, "Toc file \"%s\" is inconsistent.", TOC_FILE);
@@ -2164,21 +2318,24 @@ int main(int argc, char **argv)
   }
   
   if (COMMAND == SIMULATE || COMMAND == WRITE || COMMAND == COPY_CD) {
-    if (FULL_BURN && strcmp(DRIVER_ID, "generic-mmc-raw") != 0) {
-      message(-2, "You must use the generic-mmc-raw driver to use the "
-              "full-burn option.");
-      exitCode = 1; goto fail;
-    } else {
-      int mins = USER_CAPACITY ? USER_CAPACITY :
-        Msf(cdr->diskInfo()->capacity).min();
-      message(2, "Burning entire %d mins disc.", mins);
+    if (FULL_BURN) {
+      if (DRIVER_ID && strcmp(DRIVER_ID, "generic-mmc-raw") != 0) {
+        message(-2, "You must use the generic-mmc-raw driver to use the "
+                "full-burn option.");
+        exitCode = 1; goto fail;
+      } else {
+        int mins = USER_CAPACITY ? USER_CAPACITY :
+          Msf(cdr->diskInfo()->capacity).min();
+        message(2, "Burning entire %d mins disc.", mins);
+      }
     }
-    cdr->userCapacity(USER_CAPACITY);
     cdr->fullBurn(FULL_BURN);
+    cdr->userCapacity(USER_CAPACITY);
   }
 
   if (COMMAND == COPY_CD) {
-    if (SOURCE_SCSI_DEVICE != NULL) {
+      if (SOURCE_SCSI_DEVICE != NULL && 
+          strcmp(SCSI_DEVICE, SOURCE_SCSI_DEVICE) != 0) {
       delSrcDevice = 1;
       srcCdr = setupDevice(READ_CD, SOURCE_SCSI_DEVICE, SOURCE_DRIVER_ID,
 			 1, 1, 0, 0, 0);
@@ -2218,7 +2375,7 @@ int main(int argc, char **argv)
     break;
 
   case DRIVE_INFO:
-    showDriveInfo(cdr->driveInfo(1));
+    showDriveInfo(cdr->driveInfo(true));
     break;
 
   case SHOW_TOC:
@@ -2266,6 +2423,7 @@ int main(int argc, char **argv)
     }
     cdr->subChanReadMode(READ_SUBCHAN_MODE);
     cdr->rawDataReading(READ_RAW);
+    cdr->mode2Mixed(MODE2_MIXED);
     cdr->fastTocReading(1);
     cdr->taoSource(TAO_SOURCE);
     if (TAO_SOURCE_ADJUST >= 0)
@@ -2323,6 +2481,7 @@ int main(int argc, char **argv)
 
     cdr->subChanReadMode(READ_SUBCHAN_MODE);
     cdr->rawDataReading(READ_RAW);
+    cdr->mode2Mixed(MODE2_MIXED);
     cdr->fastTocReading(FAST_TOC);
     cdr->taoSource(TAO_SOURCE);
     if (TAO_SOURCE_ADJUST >= 0)
@@ -2376,6 +2535,7 @@ int main(int argc, char **argv)
 
     cdr->subChanReadMode(READ_SUBCHAN_MODE);
     cdr->rawDataReading(READ_RAW);
+    cdr->mode2Mixed(MODE2_MIXED);
     cdr->taoSource(TAO_SOURCE);
     if (TAO_SOURCE_ADJUST >= 0)
       cdr->taoSourceAdjust(TAO_SOURCE_ADJUST);
@@ -2395,11 +2555,6 @@ int main(int argc, char **argv)
     else {
       cdr->rezeroUnit(0);
 
-#if defined(HAVE_SETEUID) && defined(HAVE_SETEGID)
-      seteuid(getuid());
-      setegid(getgid());
-#endif
-
       if (WITH_CDDB) {
 	if (readCddb(toc) == 0) {
 	  message(2, "CD-TEXT data was added to toc-file.");
