@@ -1,7 +1,7 @@
-/* @(#)scsi-linux-ata.c	1.6 04/05/20 Copyright 2002 J. Schilling */
+/* @(#)scsi-linux-ata.c	1.7 04/06/12 Copyright 2002 J. Schilling */
 #ifndef lint
 static	char ata_sccsid[] =
-	"@(#)scsi-linux-ata.c	1.6 04/05/20 Copyright 2002 J. Schilling";
+	"@(#)scsi-linux-ata.c	1.7 04/06/12 Copyright 2002 J. Schilling";
 #endif
 /*
  *	Interface for Linux generic SCSI implementation (sg).
@@ -46,7 +46,7 @@ static	char ata_sccsid[] =
 
 #ifdef	USE_ATA
 
-LOCAL	char	_scg_atrans_version[] = "scsi-linux-ata.c-1.6";	/* The version for ATAPI transport*/
+LOCAL	char	_scg_atrans_version[] = "scsi-linux-ata.c-1.7";	/* The version for ATAPI transport*/
 
 LOCAL	char *	scgo_aversion	__PR((SCSI *scgp, int what));
 LOCAL	int	scgo_ahelp	__PR((SCSI *scgp, FILE *f));
@@ -201,8 +201,8 @@ scgo_aopen(scgp, device)
 	if (scgp->overbose)
 		error("Warning: Using ATA Packet interface.\n");
 	if (scgp->overbose) {
-		error("Warning: The related libscg interface code is in pre alpha.\n");
-		error("Warning: There may be fatal problems.\n");
+		error("Warning: The related Linux kernel interface code seems to be unmaintained.\n");
+		error("Warning: There is absolutely NO DMA, operations thus are slow.\n");
 	}
 
 	log(("\n<<<<<<<<<<<<<<<<  LOGGING ON >>>>>>>>>>>>>>>>>\n"));
@@ -267,7 +267,7 @@ openbydev:
 			starget,
 			slun;
 
-		f = open(device, O_RDONLY | O_NONBLOCK);
+		f = open(device, O_RDWR | O_NONBLOCK);
 
 		if (f < 0) {
 			if (scgp->errstr)
@@ -283,6 +283,9 @@ openbydev:
 	return (nopen);
 }
 
+#include <glob.h>
+
+
 LOCAL int
 scan_internal(scgp, nopen)
 	SCSI	*scgp;
@@ -293,118 +296,62 @@ scan_internal(scgp, nopen)
 	int	schilly_bus,
 		target,
 		lun;
-	char	device[128];
+	char	*device;
+        glob_t globbuf;
+
 	/*
 	 * try always with devfs
 	 * unfortunatelly the solution with test of existing
 	 * of '/dev/.devfsd' don't work, because it root.root 700
 	 * and i don't like run suid root
 	 */
-	BOOL	DEVFS = TRUE;
+	BOOL	DEVFS = FALSE;
 
-	if (DEVFS) {
-		for (i = 0; ; i++) {
-			sprintf(device, "/dev/cdroms/cdrom%i", i);
-			if ((f = open(device, O_RDONLY | O_NONBLOCK)) < 0) {
-				if (errno != ENOENT && errno != ENXIO && errno != ENODEV && errno != EACCES) {
-					if (scgp->debug > 4) {
-						js_fprintf((FILE *) scgp->errfile,
-						"try open(%s) return %i, errno %i, cancel\n", device, f, errno);
-					}
-					return (-2);
-				} else if (errno == ENOENT || errno == ENODEV) {
-					if (scgp->debug > 4) {
-						js_fprintf((FILE *) scgp->errfile,
-						"try open(%s) return %i, errno %i\n", device, f, errno);
-					}
-					if (0 == i) {
-						DEVFS = FALSE;
-						if (scgp->debug > 4) {
-							js_fprintf((FILE *) scgp->errfile,
-							"DEVFS not detected, continuing with old dev\n");
-						}
-					}
-					break;
-				}
+	glob("/dev/cdroms/cdrom*", 
+	     GLOB_NOSORT, 
+	     NULL, &globbuf);
+	glob("/dev/hd[a-z]", 
+	     GLOB_NOSORT|GLOB_APPEND, 
+	     NULL, &globbuf);
+	/*glob("/dev/scd*",  
+	     GLOB_NOSORT|GLOB_APPEND, 
+	     NULL, &globbuf);*/
+
+	for (i = 0; globbuf.gl_pathv && globbuf.gl_pathv[i] != NULL ; i++) {
+		device = globbuf.gl_pathv[i];
+		if ((f = open(device, O_RDWR | O_NONBLOCK)) < 0) {
+			if (errno != ENOENT && errno != ENXIO && errno != ENODEV && errno != EACCES) {
 				if (scgp->debug > 4) {
-					if (errno == EACCES) {
-						js_fprintf((FILE *) scgp->errfile,
-						"errno (EACCESS), you don't have the needed rights for %s\n",
-						device);
-					}
 					js_fprintf((FILE *) scgp->errfile,
-					"try open(%s) return %i, errno %i, trying next cdrom\n",
-					device, f, errno);
+						   "try open(%s) return %i, errno %i, cancel\n", device, f, errno);
 				}
+				globfree(&globbuf);
+				return (-2);
+			} 
+			if (scgp->debug > 4) {
+				if (errno == EACCES) {
+					js_fprintf((FILE *) scgp->errfile,
+						   "errno (EACCESS), you don't have the needed rights for %s\n",
+						   device);
+				}
+				js_fprintf((FILE *) scgp->errfile,
+					   "try open(%s) return %i, errno %i, trying next cdrom\n",
+					   device, f, errno);
+			}
+		} else {
+			if (scgp->debug > 4) {
+				js_fprintf((FILE *) scgp->errfile,
+					   "try open(%s) return %i errno %i calling sg_mapdev(...)\n",
+					   device, f, errno);
+			}
+			if (sg_amapdev(scgp, f, device, &schilly_bus, &target, &lun)) {
+				(++(*nopen));
 			} else {
-				if (scgp->debug > 4) {
-					js_fprintf((FILE *) scgp->errfile,
-					"try open(%s) return %i errno %i calling sg_mapdev(...)\n",
-					device, f, errno);
-				}
-				if (sg_amapdev(scgp, f, device, &schilly_bus, &target, &lun)) {
-					(++(*nopen));
-				} else {
-					close(f);
-				}
-			}
-		}
-	}
-	if (!DEVFS) {
-		/* for /dev/sr0 - /dev/sr? */
-		for (i = 0; ; i++) {
-			sprintf(device, "/dev/sr%i", i);
-			if ((f = open(device, O_RDONLY | O_NONBLOCK)) < 0) {
-				if (errno != ENOENT && errno != ENXIO && errno != ENODEV && errno != EACCES) {
-					if (scgp->debug > 4) {
-						js_fprintf((FILE *) scgp->errfile,
-						"try open(%s) return %i, errno %i, cancel\n",
-						device, f, errno);
-					}
-					return (-2);
-				} else if (errno == ENOENT || errno == ENODEV) {
-					break;
-				}
-			} else {
-				if (sg_amapdev(scgp, f, device, &schilly_bus, &target, &lun)) {
-					(++(*nopen));
-				} else {
-					close(f);
-				}
-			}
-		}
-
-		/* for /dev/hda - /dev/hdz */
-		for (i = 'a'; i <= 'z'; i++) {
-			sprintf(device, "/dev/hd%c", i);
-			if ((f = open(device, O_RDONLY | O_NONBLOCK)) < 0) {
-				if (errno != ENOENT && errno != ENXIO && errno != EACCES) {
-					if (scgp->debug > 4) {
-						js_fprintf((FILE *) scgp->errfile,
-						"try open(%s) return %i, errno %i, cancel\n",
-						device, f, errno);
-					}
-					return (-2);
-				} else if (errno == ENOENT || errno == ENODEV) {
-					break;
-				}
-			} else {
-				/* ugly hack, make better, when you can. Alex */
-				if (0 > ioctl(f, CDROM_DRIVE_STATUS, CDSL_CURRENT)) {
-					if (scgp->debug > 4) {
-						js_fprintf((FILE *) scgp->errfile,
-						"%s is not a cdrom, skipping\n",
-						device);
-					}
-					close(f);
-				} else if (sg_amapdev(scgp, f, device, &schilly_bus, &target, &lun)) {
-					(++(*nopen));
-				} else {
-					close(f);
-				}
+				close(f);
 			}
 		}
 	}
+	globfree(&globbuf);
 	return (0);
 }
 
@@ -689,7 +636,7 @@ sg_amapdev(scgp, f, device, schillybus, target, lun)
 		return (FALSE);
 
 	result = lstat(device, &buf);
-	if (!result && S_ISLNK(buf.st_mode)) {
+	if (0 && !result && S_ISLNK(buf.st_mode)) {
 		result = readlink(device, tmp, LOCAL_MAX_PATH);
 		if (result > 0 && result < LOCAL_MAX_PATH) {
 			tmp[result] = '\0';
