@@ -1,4 +1,4 @@
-/* @(#)niread.c	1.11 03/09/23 Copyright 1986, 1996-2003 J. Schilling */
+/* @(#)niread.c	1.12 04/08/08 Copyright 1986, 1996-2003 J. Schilling */
 /*
  *	Non interruptable read
  *
@@ -20,7 +20,7 @@
  * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "io.h"
+#include "schilyio.h"
 #include <errno.h>
 
 EXPORT int
