@@ -73,8 +73,8 @@ CdDevice::CdDevice(const char* dev, const char *vendor, const char *product)
   vendor_ = vendor;
   product_ = product;
   
-  driverId_ = 0; // undefined
-  options_ = 0;
+  driverId_ = 0;
+  driverOptions_ = 0;
 
   deviceType_ = CD_R;
 
@@ -139,7 +139,7 @@ char *CdDevice::settingString() const
 
   s += ",";
 
-  sprintf(buf, "0x%lx", options_);
+  sprintf(buf, "0x%lx", driverOptions_);
   s += buf;
 
   return strdupCC(s.c_str());
@@ -171,7 +171,7 @@ int CdDevice::autoSelectDriver()
 
   if (driverName) {
     driverId_ = driverName2Id(driverName);
-    options_ = options;
+    driverOptions_ = options;
 
   } else {
     bool r_cdr, w_cdr, r_cdrw, w_cdrw;
@@ -187,7 +187,7 @@ int CdDevice::autoSelectDriver()
       if (w_cdrw) deviceType_ = CD_RW;
     } else {
       driverId_ = DRIVER_ID_DEFAULT;
-      options_ = 0;
+      driverOptions_ = 0;
     }
     if (sif) delete sif;
   }
@@ -233,7 +233,8 @@ int CdDevice::updateStatus()
 	newStatus = DEV_NO_DISK;
 	break;
       case 3:
-	newStatus = DEV_FAULT;
+        // Most likely a timeout error.
+	newStatus = DEV_BUSY;
 	break;
       }
     }
@@ -337,12 +338,32 @@ void CdDevice::deviceType(DeviceType t)
 
 unsigned long CdDevice::driverOptions() const
 {
-  return options_;
+  return driverOptions_;
 }
 
 void CdDevice::driverOptions(unsigned long o)
 {
-  options_ = o;
+  driverOptions_ = o;
+}
+
+bool CdDevice::ejectCd(bool load)
+{
+  bool success = false;
+
+  if (!scsiIf_)
+    createScsiIf();
+
+  if (scsiIf_) {
+    CdrDriver* driver = CdrDriver::createDriver(driverName(driverId_),
+                                                driverOptions_,
+                                                scsiIf_);
+
+    int ret = driver->loadUnload((load ? 0 : 1));
+    success = (ret == 0);
+    delete(driver);
+  }
+
+  return success;
 }
 
 // Starts a 'cdrdao' for recording given toc. Returns false if an
@@ -373,7 +394,10 @@ bool CdDevice::recordDao(Gtk::Window& parent, TocEdit *tocEdit, int simulate,
     return false;
   }
 
-  if (!tocEdit->toc()->write(fd)) {
+  // Write out temporary toc file containing all the converted wav
+  // files (don't want to rely on cdrdao doing the mp3->wav
+  // translation, besides it's already been done).
+  if (!tocEdit->toc()->write(fd, true)) {
     close(fd);
     message(-2, _("Cannot write temporary toc-file."));
     return false;
@@ -422,7 +446,7 @@ bool CdDevice::recordDao(Gtk::Window& parent, TocEdit *tocEdit, int simulate,
   args[n++] = (char*)dev_.c_str();
 
   if (driverId_ > 0) {
-    sprintf(drivername, "%s:0x%lx", driverName(driverId_), options_);
+    sprintf(drivername, "%s:0x%lx", driverName(driverId_), driverOptions_);
     args[n++] = "--driver";
     args[n++] = drivername;
   }
@@ -454,7 +478,7 @@ bool CdDevice::recordDao(Gtk::Window& parent, TocEdit *tocEdit, int simulate,
     action_ = A_RECORD;
 
     if (process_->commFd() >= 0) {
-        Glib::signal_io().connect(bind(slot(*this, &CdDevice::updateProgress),
+        Glib::signal_io().connect(bind(mem_fun(*this, &CdDevice::updateProgress),
                                        process_->commFd()),
                                   process_->commFd(),
                                   Glib::IO_IN | Glib::IO_HUP);
@@ -502,7 +526,7 @@ void CdDevice::progress(int *status, int *totalTracks, int *track,
 // Starts a 'cdrdao' for reading whole cd.
 // Return: 0: OK, process succesfully launched
 //         1: error occured
-int CdDevice::extractDao(Project& parent, const char *tocFileName,
+int CdDevice::extractDao(Gtk::Window& parent, const char *tocFileName,
                          int correction, int readSubChanMode)
 {
   char *args[30];
@@ -553,7 +577,7 @@ int CdDevice::extractDao(Project& parent, const char *tocFileName,
   args[n++] = (char*)dev_.c_str();
 
   if (driverId_ > 0) {
-    sprintf(drivername, "%s:0x%lx", driverName(driverId_), options_);
+    sprintf(drivername, "%s:0x%lx", driverName(driverId_), driverOptions_);
     args[n++] = "--driver";
     args[n++] = drivername;
   }
@@ -587,7 +611,7 @@ int CdDevice::extractDao(Project& parent, const char *tocFileName,
     action_ = A_READ;
 
     if (process_->commFd() >= 0) {
-        Glib::signal_io().connect(bind(slot(*this, &CdDevice::updateProgress),
+        Glib::signal_io().connect(bind(mem_fun(*this, &CdDevice::updateProgress),
                                        process_->commFd()),
                                   process_->commFd(),
                                   Glib::IO_IN | Glib::IO_PRI |
@@ -611,7 +635,7 @@ void CdDevice::abortDaoReading()
 // Starts a 'cdrdao' for duplicating a CD.
 // Return: 0: OK, process succesfully launched
 //         1: error occured
-int CdDevice::duplicateDao(Project& parent, int simulate, int multiSession,
+int CdDevice::duplicateDao(Gtk::Window& parent, int simulate, int multiSession,
                            int speed, int eject, int reload, int buffer,
                            int onthefly, int correction, int readSubChanMode, 
 			   CdDevice *readdev)
@@ -695,7 +719,7 @@ int CdDevice::duplicateDao(Project& parent, int simulate, int multiSession,
   args[n++] = (char*)dev_.c_str();
 
   if (driverId_ > 0) {
-    sprintf(drivername, "%s:0x%lx", driverName(driverId_), options_);
+    sprintf(drivername, "%s:0x%lx", driverName(driverId_), driverOptions_);
     args[n++] = "--driver";
     args[n++] = drivername;
   }
@@ -743,7 +767,7 @@ int CdDevice::duplicateDao(Project& parent, int simulate, int multiSession,
     action_ = A_DUPLICATE;
 
     if (process_->commFd() >= 0) {
-        Glib::signal_io().connect(bind(slot(*this, &CdDevice::updateProgress),
+        Glib::signal_io().connect(bind(mem_fun(*this, &CdDevice::updateProgress),
                                        process_->commFd()),
                                   process_->commFd(),
                                   Glib::IO_IN | Glib::IO_HUP);
@@ -766,7 +790,7 @@ void CdDevice::abortDaoDuplication()
 // Starts a 'cdrdao' for blanking a CD.
 // Return: 0: OK, process succesfully launched
 //         1: error occured
-int CdDevice::blank(Project* parent, int fast, int speed, int eject,
+int CdDevice::blank(Gtk::Window* parent, int fast, int speed, int eject,
                     int reload)
 {
   char *args[20];
@@ -821,7 +845,7 @@ int CdDevice::blank(Project* parent, int fast, int speed, int eject,
   args[n++] = (char*)dev_.c_str();
 
   if (driverId_ > 0) {
-    sprintf(drivername, "%s:0x%lx", driverName(driverId_), options_);
+    sprintf(drivername, "%s:0x%lx", driverName(driverId_), driverOptions_);
     args[n++] = "--driver";
     args[n++] = drivername;
   }
@@ -849,7 +873,7 @@ int CdDevice::blank(Project* parent, int fast, int speed, int eject,
     action_ = A_BLANK;
 
     if (process_->commFd() >= 0) {
-        Glib::signal_io().connect(bind(slot(*this, &CdDevice::updateProgress),
+        Glib::signal_io().connect(bind(mem_fun(*this, &CdDevice::updateProgress),
                                        process_->commFd()),
                                   process_->commFd(),
                                   Glib::IO_IN | Glib::IO_HUP);
